import { routerRedux } from 'dva/router';
import { isString } from 'lodash';
import { logout } from '../../services/main';
import { fetchDomains, switchDomain, currentDomain } from '../../services/domain';
import { cookie } from '../../utils/config';
import { getMenus, getModuleInfo } from '../../data/modules';
import { setCookie, delCookie, fullPath, getCookie } from '../../utils/helper';

const createMenu = async (configure) => {
  const menu = {
    name: configure.name,
    icon: configure.icon,
    text: configure.showName,
  };
  if (!menu.children) {
    menu.children = [];
  }
  if (configure.children) {
    for (const child of configure.children) {
      menu.children.push(await createMenu(child));
    }
  }
  if (configure.modules) {
    for (const module of configure.modules) {
      let info = module;
      if (isString(module)) {
        info = await getModuleInfo(module);
      }
      menu.children.push({
        name: info.name,
        icon: info.icon,
        text: info.showName,
      });
    }
  }
  return menu;
};

const makeIcon = (menu) => {
  const children = menu.children;
  if (!children || children.length === 0) {
    menu.icon = 'file'; // eslint-disable-line no-param-reassign
  } else {
    menu.icon = 'folder'; // eslint-disable-line no-param-reassign
    for (const child of children) {
      makeIcon(child);
    }
  }
};

const createMenus = async (menusConfigure) => {
  const menus = [];
  if (menusConfigure) {
    for (const configure of menusConfigure) {
      menus.push(await createMenu(configure));
    }
  }
  for (const menu of menus) {
    makeIcon(menu);
  }
  return menus;
};

// const createMenus = () => {
//   const menus = [];
//   const makePath = (_menus, path) => {
//     if (path.length === 0) {
//       return null;
//     }
//     const [cur, ...rest] = path;
//     const find = _menus.filter(menu => menu.name === `${cur}`).pop();
//     if (find) {
//       if (rest.length === 0) {
//         return find;
//       } else {
//         return makePath(find.children, rest);
//       }
//     } else {
//       let container = _menus;
//       for (const id of path) {
//         const m = getModule(id);
//         const menu = {
//           name: `${m.id}`,
//           text: m.name,
//           children: [],
//         };
//         container.push(menu);
//         container = menu.children;
//       }
//     }
//   };
//   const makeIcon = (menu) => {
//     const children = menu.children;
//     if (children.length === 0) {
//       menu.icon = 'file'; // eslint-disable-line no-param-reassign
//     } else {
//       menu.icon = 'folder'; // eslint-disable-line no-param-reassign
//       for (const child of children) {
//         makeIcon(child);
//       }
//     }
//   };
//   foreachModule((module) => {
//     const path = getPath(module.id);
//     makePath(menus, path);
//   });
//   for (const menu of menus) {
//     makeIcon(menu);
//   }
//   return menus;
// };

export default {

  namespace: 'main',

  state: {
    user: '',
    domain: '',
    domainList: [],
    menus: [],
  },

  subscriptions: {
    setup({ dispatch, history }) {  // eslint-disable-line
    },
  },

  effects: {
    *fetchDomain(action, { put, call }) {
      let domain = getCookie(cookie.domainName);
      if (!domain) {
        const { path, name } = yield call(currentDomain);
        setCookie(cookie.domainPath, path);
        setCookie(cookie.domainName, name);
        domain = name;
      }
      yield put({ type: 'switchDomainSuccess', payload: domain });
    },
    *fetchDomains(action, { put, call }) {
      const list = yield call(fetchDomains, '/', true);
      yield put({ type: 'queryDomainsSuccess', payload: list });
    },
    *switchDomain({ payload: domainId }, { put, call }) {
      yield call(switchDomain, domainId);
      const { path, name } = yield call(currentDomain);
      setCookie(cookie.domainPath, path);
      setCookie(cookie.domainName, name);
      yield put({ type: 'switchDomainSuccess', payload: name });
      yield put(routerRedux.push(fullPath('/main')));
    },
    *fetchUser(action, { put }) {
      const user = getCookie(cookie.userName);
      if (!user) {
        yield put(routerRedux.push(fullPath('/login')));
      }
      yield put({ type: 'queryUserSuccess', payload: user });
    },
    *fetchModules(action, { put, call }) {
      const configures = yield call(getMenus);
      const menus = yield call(createMenus, configures);
      yield put({ type: 'queryMenusSuccess', payload: menus });
    },
    *logout(action, { put, call }) {
      yield call(logout);
      delCookie(cookie.domainName);
      delCookie(cookie.userName);
      delCookie(cookie.domainPath);
      delCookie(cookie.userId);
      delCookie(cookie.token);
      yield put(routerRedux.push(fullPath('/login')));
    },
  },

  reducers: {
    queryUserSuccess(state, { payload: user }) {
      return {
        ...state,
        user,
      };
    },
    queryDomainsSuccess(state, { payload: list }) {
      return {
        ...state,
        domainList: list,
      };
    },
    switchDomainSuccess(state, { payload: domain }) {
      return {
        ...state,
        domain,
      };
    },
    queryMenusSuccess(state, { payload: menus }) {
      return {
        ...state,
        menus,
      };
    },
  },

};
